package cn.com.ty.lift.quan.news.controller;

import cn.com.ty.lift.quan.news.dao.entity.QuanNews;
import cn.com.ty.lift.quan.news.dao.entity.model.QuanNewsReq;
import cn.com.ty.lift.quan.news.service.impl.QuanNewsServiceImpl;
import cn.com.xwy.boot.web.dto.RestResponse;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;


/**
 * <p>
 * 电梯圈新闻公告 前端控制器
 * </p>
 *
 * @author wang-hai-cheng
 * @since 2019-12-02
 */
@RestController
@AllArgsConstructor
@RequestMapping("/quan-news")
public class QuanNewsController {
    private final QuanNewsServiceImpl quanNewsService;

    @PostMapping("/list")
    /**
     * @description 分页展示置顶新闻+普通新闻
     * @date 2019/12/2 15:39
     * @param [页码, 页大小]
     * @return cn.com.xwy.boot.web.dto.RestResponse
     */
    public RestResponse list(@RequestBody QuanNewsReq quanNews) {
        return quanNewsService.news(quanNews.getPageNum(), quanNews.getPageSize(), quanNews.getHaveTops());
    }

    @PostMapping("/top")
    /**
     * @description 置顶
     * @date 2019/12/2 15:40
     * @param [新闻id]
     * @return cn.com.xwy.boot.web.dto.RestResponse
     */
    public RestResponse top(@RequestBody QuanNews quanNews) {
        return quanNewsService.top(quanNews.getId());
    }

    @PostMapping("/top/cancel")
    /**
     * @description 取消置顶
     * @date 2019/12/3 13:28
     * @param [新闻id]
     * @return cn.com.xwy.boot.web.dto.RestResponse
     */
    public RestResponse cancelTop(@RequestBody QuanNews quanNews) {
        return quanNewsService.cancelTop(quanNews.getId());
    }

    @GetMapping("/{id}")
    /**
     * @description 根据id获取新闻
     * @date 2019/12/2 15:48
     * @param [新闻id]
     * @return cn.com.xwy.boot.web.dto.RestResponse
     */
    public RestResponse news(@PathVariable Long id) {
        QuanNews byId = quanNewsService.getById(id);
        if (null != byId) {
            return RestResponse.ok(byId, "1", "查询成功");
        } else {
            return RestResponse.ok(null, "9", "无数据");
        }
    }

    @PostMapping("/add")
    /**
     * @description 新增新闻
     * @date 2019/12/2 15:48
     * @param [新闻标题, 新闻内容, 视频文件, 发布人id]
     * @return org.springframework.http.ResponseEntity
     */
    public RestResponse add(@RequestParam String title, @RequestParam String content, @RequestParam Long releaseId, @RequestParam String releaseUser, @RequestParam MultipartFile videoFile, @RequestParam Integer isTop) {
        QuanNews quanNews = new QuanNews();
        quanNews.setTitle(title)
                .setContent(content)
                .setReleaseId(releaseId)
                .setReleaseUser(releaseUser);
        return quanNewsService.addNew(quanNews, videoFile, isTop);
    }

    @PostMapping("/update")
    /**
     * @description 根据id更新新闻信息
     * @date 2019/12/2 15:49
     * @param [新闻id, 新闻标题, 新闻内容]
     * @return org.springframework.http.ResponseEntity
     */
    public RestResponse updateNews(@RequestParam Long id, @RequestParam String title, @RequestParam String content, @RequestParam Long releaseId, @RequestParam String releaseUser, @RequestParam MultipartFile videoFile) {
        QuanNews quanNews = new QuanNews();
        quanNews.setId(id)
                .setTitle(title)
                .setContent(content)
                .setReleaseId(releaseId)
                .setReleaseUser(releaseUser);
        return quanNewsService.updateNews(quanNews, videoFile);
    }

    @PostMapping("/delete")
    /**
     * @description 根据id删除新闻
     * @date 2019/12/2 15:49
     * @param [新闻id]
     * @return org.springframework.http.ResponseEntity
     */
    public RestResponse delete(@RequestBody QuanNews quanNews) {
        return quanNewsService.delete(quanNews.getId());
    }
}